from ._flt1002 import FLT1002 as _FLT1002
from ._flt1003 import FLT1003 as _FLT1003
from ._flt1004 import FLT1004 as _FLT1004
from ._flt1005 import FLT1005 as _FLT1005
from ._flt1006 import FLT1006 as _FLT1006
from ._flt1007 import FLT1007 as _FLT1007

FLIGHT_DESCRIPTIONS = {
    "1002": _FLT1002,
    "1003": _FLT1003,
    "1004": _FLT1004,
    "1005": _FLT1005,
    "1006": _FLT1006,
    "1007": _FLT1007,
}
from ._sensor import SENSOR_DESCRIPTIONS



from ._flight import FlightDescription, FlightSegment, Report

# 构造 Flight 1003
FLT1003 = FlightDescription(
    flight_number="1003",
    date="29-Jun-2020",
    doy="181/366",
    compensations=[],  # 未提及补偿动作
    survey_lines=["Eastern Ontario & Renfrew free-fly at 400m & 800m"],  # 巡查线描述
    issues=[
        Report(
            "Mag 2 periodically dropped signal at ~180 degree headings during pitch/roll maneuvers",
            affected_lines=["1003.01"],
        ),
    ],
)

# 该航班的段落数据
segments_data_1003 = [
    (49820.00, 50713.00, "1003.01", "Takeoff - Eastern Ontario Free-Fly"),
    (50713.00, 54497.00, "1003.02", "Eastern Ontario Free-Fly 400m"),
    (54497.00, 54639.00, "1003.03", "Climb to 800m"),
    (54639.00, 59475.00, "1003.04", "Eastern Ontario Free-Fly 800m"),
    (59475.00, 59926.00, "1003.05", "Transit at 800m"),
    (59926.00, 60105.00, "1003.06", "Descend to 400m"),
    (60105.00, 60243.00, "1003.07", "Transit to Renfrew Free-Fly"),
    (60243.00, 64586.00, "1003.08", "Renfrew Free-Fly 400m"),
    (64586.00, 64763.00, "1003.09", "Climb to 800m"),
    (64763.00, 69252.00, "1003.10", "!!!HOLD-OUT TESTING DATA!!!"),
    (69252.00, 70311.00, "1003.11", "Transit to base"),
]

for start_time, end_time, line_number, description in segments_data_1003:
    FLT1003.add_segment(FlightSegment(start_time, end_time, line_number, description))



from ._flight import FlightDescription, FlightSegment, Report  # 构造 Flight 1005

FLT1005 = FlightDescription(
    flight_number="1005",
    date="02-Jul-2020",
    doy="184/366",
    compensations=[],  # 未提及补偿动作
    survey_lines=[
        "Perth mini-survey (within Eastern Ontario) flown at 800m, flight 2 of 2"
    ],  # 巡查线描述
    issues=[
        Report("Split into two flights due to weather (low altitude clouds)"),
    ],
)

# 该航班的段落数据
segments_data_1005 = [
    (45788.00, 48085.00, "1005.01", "Takeoff - Transit to Perth Mini-Survey Area"),
    (48085.00, 48666.66, "4011.00", "Perth Survey Line"),
    (48847.12, 49498.52, "4010.00", "Perth Survey Line"),
    (49659.65, 50241.60, "4009.00", "Perth Survey Line"),
    (50482.41, 51080.55, "4008.00", "Perth Survey Line"),
    (51260.85, 51848.30, "4007.00", "Perth Survey Line"),
    (51998.63, 52604.72, "4006.00", "!!!HOLD-OUT TESTING DATA!!!"),
    (52740.76, 53315.22, "4005.00", "!!!HOLD-OUT TESTING DATA!!!"),
    (53471.63, 54063.86, "4004.00", "Perth Survey Line"),
    (54232.63, 54811.78, "4003.00", "Perth Survey Line"),
    (55006.06, 55605.59, "4002.00", "Perth Survey Line"),
    (55605.59, 56710.00, "1005.02", "Transit to base"),
]

for start_time, end_time, line_number, description in segments_data_1005:
    FLT1005.add_segment(FlightSegment(start_time, end_time, line_number, description))



from sgl2020.descriptions._flight import FlightDescription, FlightSegment, Report

# 构造 Flight 1007
FLT1007 = FlightDescription(
    flight_number="1007",
    date="07-Jul-2020",
    doy="189/366",
    compensations=[],  # 未提及补偿动作
    survey_lines=[
        "Free-fly in Perth mini-survey at 800m",
        "Free-fly in Eastern Ontario & Renfrew at 400m",
    ],  # 巡查线描述
    issues=[],  # 未提及具体问题
)

# 该航班的段落数据
segments_data_1007 = [
    (47070.00, 48024.00, "1007.01", "Transit to Perth Mini-Survey Area"),
    (48024.00, 51880.00, "1007.02", "Free-Fly at 800m within Perth Mini-Survey Area"),
    (51880.00, 52050.00, "1007.03", "Descent/Transit to 400m Eastern Ontario Free-Fly"),
    (52050.00, 57330.00, "1007.04", "!!!HOLD-OUT TESTING DATA!!!"),
    (57330.00, 57770.00, "1007.05", "Transit to Renfrew Free-Fly"),
    (57770.00, 63010.00, "1007.06", "Free-Fly at 400m within Renfrew Area"),
    (63010.00, 63800.00, "1007.07", "Transit to base"),
]

for start_time, end_time, line_number, description in segments_data_1007:
    FLT1007.add_segment(FlightSegment(start_time, end_time, line_number, description))



from ._flight import FlightDescription, FlightSegment, Report

# 构造 Flight 1006
FLT1006 = FlightDescription(
    flight_number="1006",
    date="06-Jul-2020",
    doy="188/366",
    compensations=[
        "Compensation maneuvers in FOM at 10,000ft (3048m), 13,000ft (3962m), 17,000ft (5181m)",
        "Compensation maneuvers in Eastern Ontario at 400m",
    ],  # 补偿动作的描述
    survey_lines="",  # 未提及巡查线
    issues=[
        Report(
            "Mag 2 periodically dropped signal at ~180 degree headings during pitch/roll maneuvers",
            affected_lines=[
                "1006.02",
                "1006.03",
                "1006.04",
                "1006.06",
                "1006.07",
                "1006.08",
            ],
        ),
        Report(
            "Limited pilot comments written during flight due to challenging maneuvers"
        ),
    ],
)

# 该航班的段落数据
segments_data_1006 = [
    (46100.00, 47222.00, "1006.01", "Takeoff - Compensation area"),
    (47222.00, 48213.00, "1006.02", "!!!HOLD-OUT TESTING DATA!!!"),
    (48213.00, 49000.00, "1006.03", "Climb to 17,000ft"),
    (49000.00, 53286.00, "1006.04", "Compensation maneuvers at 17,000ft"),
    (53286.00, 53855.00, "1006.05", "Descent to 10,000ft"),
    (53855.00, 54510.00, "1006.06", "Compensation maneuvers at 10,000ft"),
    (54510.00, 55770.00, "1006.07", "Transit/Descent to Eastern Ontario"),
    (
        55770.00,
        56609.00,
        "1006.08",
        "Compensation maneuvers in Eastern Ontario at 400m",
    ),
    (56609.00, 57922.00, "1006.09", "Transit to base"),
]

for start_time, end_time, line_number, description in segments_data_1006:
    FLT1006.add_segment(FlightSegment(start_time, end_time, line_number, description))



from prettytable import PrettyTable
from typing_extensions import List


class FlightDescription:
    def __init__(
        self,
        flight_number: str,
        date: str,
        doy: str,
        compensations: List[str] = [],
        survey_lines: List[str] = [],
        issues: List["Report"] = [],
    ):
        self.flight_number = flight_number
        self.date = date
        self.doy = doy
        self.compensations = compensations
        self.survey_lines = survey_lines
        self.issues = issues
        self.segments: List["FlightSegment"] = []

    def add_segment(self, segment: "FlightSegment") -> None:
        self.segments.append(segment)

    @property
    def lines(self) -> List[str]:
        return list(set([seg.line_number for seg in self.segments]))

    def describe(self) -> None:
        print(f"Flight {self.flight_number}")
        print(f"Date: {self.date} (Day of Year: {self.doy})")
        print("Compensations:")
        if self.compensations:
            for compensation in self.compensations:
                print(f"  - {compensation}")
        else:
            print("  - No Description")
        print("Survey Lines:")
        if self.survey_lines:
            for line in self.survey_lines:
                print(f"  - {line}")
        else:
            print("  - No Description")
        print("Issues:")
        if self.issues:
            for issue in self.issues:
                issue.describe()
        else:
            print("  - No Description")
        print("\nFlight Segments:")
        segment_table = PrettyTable()
        segment_table.field_names = [
            "Start Time",
            "End Time",
            "Line Number",
            "Description",
        ]
        for segment in self.segments:
            segment_table.add_row(
                [
                    segment.start_time,
                    segment.end_time,
                    segment.line_number,
                    segment.description,
                ]
            )
        print(segment_table)


class FlightSegment:
    def __init__(
        self, start_time: float, end_time: float, line_number: str, description: str
    ) -> None:
        self.start_time = start_time
        self.end_time = end_time
        self.line_number = line_number
        self.description = description

    def to_csv(self) -> str:
        return f'{self.start_time},{self.end_time},{self.line_number},"{self.description}"\n'


class Report:
    def __init__(self, description: str, affected_lines: List[str] = None) -> None:
        self.description = description
        self.affected_lines = affected_lines or []

    def describe(self) -> None:
        print(f"  - {self.description}")
        if self.affected_lines:
            print(f"    - Affected Lines: {', '.join(self.affected_lines)}")



import re

import numpy as np
import pooch
from h5py import File, is_hdf5
from pandas import DataFrame, Series
from typing_extensions import Dict, List, Literal, Self, Union

from sgl2020.descriptions import FLIGHT_DESCRIPTIONS, SENSOR_DESCRIPTIONS


class DataWarehouse:
    def __init__(self) -> None:
        self.resource = pooch.create(
            path=pooch.os_cache("sgl2020"),
            base_url="doi:10.5281/zenodo.4271803/",
            registry={
                "Flt1002_train.h5": "md5:d42e6579719e31d17115c9b8ed3e4471",
                "Flt1003_train.h5": "md5:bc0e819bebb8bc9d4814cdbce0fb911a",
                "Flt1004_train.h5": "md5:a17c1678407c02f260d7250b4e6c3a15",
                "Flt1005_train.h5": "md5:0065e0a270db7f53544ba8df247e1c12",
                "Flt1006_train.h5": "md5:fb798e26d515673deae2e1b349b9e215",
                "Flt1007_train.h5": "md5:ad29d8bb5ab8a3c97ac02b8f83b41b01",
            },
        )
        self.file_handles: Dict[str, File] = {}

    def get_file_handle(self, flight: str) -> File:
        dataset_filename = f"Flt{flight}_train.h5"
        if flight not in self.file_handles:
            dataset_filepath = self.resource.fetch(dataset_filename, progressbar=True)
            if not is_hdf5(dataset_filepath):
                raise ValueError(
                    f"{dataset_filepath} is not a valid HDF5 file or does not exist."
                )
            self.file_handles[flight] = File(dataset_filepath, "r")
        return self.file_handles[flight]

    def close(self) -> None:
        for handle in self.file_handles.values():
            handle.close()


class Sgl2020:
    def __init__(self, reset_index=False) -> None:
        self.data_warehouse = DataWarehouse()
        self.lines_seleted: List[str] = None
        self.sources_requested: List[str] = None
        self.reset_index = reset_index
        # TODO: 处理空数据，如不存在的航线、数据集内没有对应数据等
        # self.without_empty_data = False
        # TODO: 排除取出数据的时间段，如航线说明带有“HOLD-OUT”字样的
        # self.without_holdout_period = False

    def __del__(self) -> None:
        self.data_warehouse.close()

    def line(self, selected: List[Union[float, str]]) -> Self:
        lines_seleted = self._parse_lines_selected(selected)
        # self._validate_lines(lines_seleted)
        self.lines_seleted = lines_seleted
        return self

    def source(self, sources: List[str]) -> Self:
        # self._validate_sources(sources)
        self.sources_requested = sources
        return self

    def take(self) -> Dict[str, DataFrame]:
        if self.lines_seleted is None or self.sources_requested is None:
            raise ValueError("Please specify lines and sources first.")

        data = {
            line_id: self._fetch_line_data(line_id) for line_id in self.lines_seleted
        }
        return data

    FltString = Literal["1002", "1003", "1004", "1005", "1006", "1007"]
    FltInt = Literal[1002, 1003, 1004, 1005, 1006, 1007]

    @staticmethod
    def describe(
        mode: Union[Literal["flight", "sensor"], FltString, FltInt] = "sensor"
    ) -> None:
        # validate
        if mode not in ["flight", "sensor"] and str(mode) not in FLIGHT_DESCRIPTIONS:
            raise ValueError(
                f"Invalid mode: {mode}, should be one of 'flight', 'sensor' or flight id."
            )

        if mode == "flight":
            print("Flight descriptions:")
            for flight_id in FLIGHT_DESCRIPTIONS:
                FLIGHT_DESCRIPTIONS[flight_id].describe()
        elif mode == "sensor":
            SENSOR_DESCRIPTIONS.describe()
        else:  # mode is flight id
            FLIGHT_DESCRIPTIONS[str(mode)].describe()

    def _parse_lines_selected(self, selected: List[Union[float, str]]) -> List[str]:
        # 支持“*.*”选择所有航线
        # 支持“1002.*”选择1002航班的所有航线，根据数据文件,可能有非1002开头的航线
        # 支持“1002.01”这样的航线号
        # 支持1002.01这样的航线号
        # 支持“1002.01-1002.02”这样的航线号
        pattern = r"^(\d{4}\.\d{2}|\d{4}\.\*|\d{4}\.\d{2}\-\d{4}\.\d{2}|\*\.\*)$"

        ret: List[str] = []
        for line in selected:
            if isinstance(line, float):
                line = f"{line:.2f}"
            if not re.match(pattern, line):
                raise ValueError(f"Invalid line: {line}")

            if line == "*.*":
                for flight in FLIGHT_DESCRIPTIONS:
                    ret.extend(self._expand_line(flight))
            elif "*" in line:
                # 1002.* => [1002.01, 1002.02, ...]
                ret.extend(self._expand_line(self._parse_flight_from_line(line)))
            elif "-" in line:
                ret.extend(self._expand_line_range(line))
            else:
                ret.append(line)
        return list(set(ret))

    def _expand_line(self, flight: str) -> List[str]:
        lines_expanded = [
            seg.line_number for seg in FLIGHT_DESCRIPTIONS[flight].segments
        ]
        lines_expanded = list(set(lines_expanded))
        return lines_expanded

    def _expand_line_range(self, line: str) -> List[str]:
        # 1002.02-1002.05 => [1002.02, 1002.03, 1002.04, 1002.05]
        start, end = line.split("-")
        start_flight = self._parse_flight_from_line(start)
        end_flight = self._parse_flight_from_line(end)

        if start_flight != end_flight:
            raise ValueError(f"Only support line range in the same flight: {line}")

        start_line = int(start.split(".")[1])
        end_line = int(end.split(".")[1])

        lines_expanded = [
            f"{start_flight}.{line:02d}" for line in range(start_line, end_line + 1)
        ]
        return lines_expanded

    def _parse_flight_from_line(self, line: str) -> str:
        def find_flight(line: str):
            for id, flight in FLIGHT_DESCRIPTIONS.items():
                if line in flight.lines:
                    return flight.flight_number

        flight = line.split(".")[0].strip()
        if flight not in FLIGHT_DESCRIPTIONS:
            flight = find_flight(line)
        return flight

    def _fetch_line_data(self, line_id: str) -> DataFrame:
        flight_id = self._parse_flight_from_line(line_id)
        file_handle = self.data_warehouse.get_file_handle(flight_id)
        return self._read_data(file_handle, line_id)

    def _read_data(self, file_handle: File, line_id: str) -> DataFrame:
        lines = [f"{line:.2f}" for line in file_handle["line"][:]]
        mask = np.array(lines) == line_id

        index = np.where(mask)[0] if not self.reset_index else None

        line_data = {
            source: Series(file_handle[f"/{source}"][:][mask], index=index)
            for source in self.sources_requested
        }

        return DataFrame(line_data)



from prettytable import PrettyTable
from typing_extensions import Dict, List


class FieldInfo:
    def __init__(self, field: str, units: str, description: str) -> None:
        self.field = field
        self.units = units
        self.description = description


class Note:
    def __init__(self, text: str, sub_notes: List["Note"] = None):
        self.text: str = text
        self.sub_notes: List[Note] = sub_notes if sub_notes is not None else []

    def add_sub_note(self, sub_note: "Note") -> None:
        self.sub_notes.append(sub_note)

    def __str__(self, level: int = 0) -> str:
        result: str = "  " * level + "- " + self.text + "\n"
        for sub_note in self.sub_notes:
            result += sub_note.__str__(level + 1)
        return result


class SensorPosition:
    def __init__(
        self,
        sensor: str,
        description: str,
        x: float,
        y: float,
        z: float,
        flight_number: str = None,
    ) -> None:
        self.sensor = sensor
        self.description = description
        self.x = x
        self.y = y
        self.z = z
        self.flight_number = flight_number


class SensorDescription:
    def __init__(self) -> None:
        self.fields: List[FieldInfo] = []
        self.sensor_positions: List[SensorPosition] = []
        self.field_notes: Dict[str, Note] = {}

    def add_field(self, field: str, units: str, description: str) -> None:
        self.fields.append(FieldInfo(field, units, description))

    def add_sensor_position(
        self,
        sensor: str,
        description: str,
        x: float,
        y: float,
        z: float,
        flight_number: str = None,
    ) -> None:
        self.sensor_positions.append(
            SensorPosition(sensor, description, x, y, z, flight_number)
        )

    def add_field_notes(self, field, note):
        self.field_notes[field] = note

    def describe(self) -> None:
        self.describe_notes()
        field_table = PrettyTable()
        field_table.field_names = ["Field", "Units", "Description"]
        for field in self.fields:
            field_table.add_row([field.field, field.units, field.description])
        print("Field Descriptions:")
        print(field_table)

        # 显示传感器位置信息
        sensor_table = PrettyTable()
        sensor_table.field_names = [
            "Sensor",
            "Description",
            "X",
            "Y",
            "Z",
            "Flight Number",
        ]
        for sensor in self.sensor_positions:
            sensor_table.add_row(
                [
                    sensor.sensor,
                    sensor.description,
                    sensor.x,
                    sensor.y,
                    sensor.z,
                    sensor.flight_number or "Default",
                ]
            )
        print("\nSensor Positions:")
        print(sensor_table)

    def describe_notes(self) -> None:
        print("Notes on Specific Flight Data Fields:")
        for field, note in self.field_notes.items():
            print(note)


# 创建 FlightDataDescription 实例
SENSOR_DESCRIPTIONS = SensorDescription()

# 添加字段信息
fields_data = [
    ("line", "-", "line number"),
    ("flight", "-", "flight number"),
    ("year", "-", "year"),
    ("doy", "-", "day of year"),
    ("tt", "s", "fiducial seconds past midnight UTC"),
    ("utm_x", "m", "x-coordinate, WGS-84 UTM zone 18N"),
    ("utm_y", "m", "y-coordinate, WGS-84 UTM zone 18N"),
    ("utm_z", "m", "z-coordinate, GPS altitude (WGS-84)"),
    ("msl", "m", "z-coordinate, GPS altitude above EGM2008 Geoid"),
    ("lat", "deg", "latitude, WGS-84"),
    ("lon", "deg", "longitude, WGS-84"),
    ("baro", "m", "barometric altimeter"),
    ("radar", "m", "filtered radar altimeter"),
    ("topo", "m", "radar topography (WGS-84)"),
    ("dem", "m", "digital elevation model from SRTM (WGS-84)"),
    ("drape", "m", "planned survey drape (WGS-84)"),
    ("ins_pitch", "deg", "INS-computed aircraft pitch"),
    ("ins_roll", "deg", "INS-computed aircraft roll"),
    ("ins_yaw", "deg", "INS-computed aircraft yaw"),
    ("diurnal", "nT", "measured diurnal"),
    ("mag_1_c", "nT", "Mag 1: compensated magnetic field"),
    ("mag_1_lag", "nT", "Mag 1: lag-corrected magnetic field"),
    ("mag_1_dc", "nT", "Mag 1: diurnal-corrected magnetic field"),
    ("mag_1_igrf", "nT", "Mag 1: IGRF & diurnal-corrected magnetic field"),
    ("mag_1_uc", "nT", "Mag 1: uncompensated magnetic field"),
    ("mag_2_uc", "nT", "Mag 2: uncompensated magnetic field"),
    ("mag_3_uc", "nT", "Mag 3: uncompensated magnetic field"),
    ("mag_4_uc", "nT", "Mag 4: uncompensated magnetic field"),
    ("mag_5_uc", "nT", "Mag 5: uncompensated magnetic field"),
    ("mag_6_uc", "nT", "Mag 6: uncompensated magnetic field"),
    ("flux_a_x", "nT", "Flux A: fluxgate x-axis"),
    ("flux_a_y", "nT", "Flux A: fluxgate y-axis"),
    ("flux_a_z", "nT", "Flux A: fluxgate z-axis"),
    ("flux_a_t", "nT", "Flux A: fluxgate total"),
    ("flux_b_x", "nT", "Flux B: fluxgate x-axis"),
    ("flux_b_y", "nT", "Flux B: fluxgate y-axis"),
    ("flux_b_z", "nT", "Flux B: fluxgate z-axis"),
    ("flux_b_t", "nT", "Flux B: fluxgate total"),
    ("flux_c_x", "nT", "Flux C: fluxgate x-axis"),
    ("flux_c_y", "nT", "Flux C: fluxgate y-axis"),
    ("flux_c_z", "nT", "Flux C: fluxgate z-axis"),
    ("flux_c_t", "nT", "Flux C: fluxgate total"),
    ("flux_d_x", "nT", "Flux D: fluxgate x-axis"),
    ("flux_d_y", "nT", "Flux D: fluxgate y-axis"),
    ("flux_d_z", "nT", "Flux D: fluxgate z-axis"),
    ("flux_d_t", "nT", "Flux D: fluxgate total"),
    ("ogs_mag", "nT", "OGS survey diurnal-corrected, levelled, magnetic field"),
    ("ogs_alt", "m", "OGS survey, GPS altitude (WGS-84)"),
    ("ins_acc_x", "m/s^2", "INS x-acceleration"),
    ("ins_acc_y", "m/s^2", "INS y-acceleration"),
    ("ins_acc_z", "m/s^2", "INS z-acceleration"),
    ("ins_wander", "rad", "INS-computed wander angle (ccw from north)"),
    ("ins_lat", "rad", "INS-computed latitude"),
    ("ins_lon", "rad", "INS-computed longitude"),
    ("ins_alt", "m", "INS-computed altitude (WGS-84)"),
    ("ins_vn", "m/s", "INS-computed north velocity"),
    ("ins_vw", "m/s", "INS-computed west velocity"),
    ("ins_vu", "m/s", "INS-computed vertical (up) velocity"),
    ("pitch_rate", "deg/s", "avionics-computed pitch rate"),
    ("roll_rate", "deg/s", "avionics-computed roll rate"),
    ("yaw_rate", "deg/s", "avionics-computed yaw rate"),
    ("lgtl_acc", "g", "avionics-computed longitudinal (forward) acceleration"),
    ("ltrl_acc", "g", "avionics-computed lateral (starboard) acceleration"),
    ("nrml_acc", "g", "avionics-computed normal (vertical) acceleration"),
    ("tas", "m/s", "avionics-computed true airspeed"),
    ("pitot_p", "kPa", "avionics-computed pitot pressure"),
    ("static_p", "kPa", "avionics-computed static pressure"),
    ("total_p", "kPa", "avionics-computed total pressure"),
    ("cur_com_1", "A", "current sensor: aircraft radio 1"),
    ("cur_ac_hi", "A", "current sensor: air conditioner fan high"),
    ("cur_ac_lo", "A", "current sensor: air conditioner fan low"),
    ("cur_tank", "A", "current sensor: cabin fuel pump"),
    ("cur_flap", "A", "current sensor: flap motor"),
    ("cur_strb", "A", "current sensor: strobe lights"),
    ("cur_srvo_o", "A", "current sensor: INS outer servo"),
    ("cur_srvo_m", "A", "current sensor: INS middle servo"),
    ("cur_srvo_i", "A", "current sensor: INS inner servo"),
    ("cur_heat", "A", "current sensor: INS heater"),
    ("cur_acpwr", "A", "current sensor: aircraft power"),
    ("cur_outpwr", "A", "current sensor: system output power"),
    ("cur_bat_1", "A", "current sensor: battery 1"),
    ("cur_bat_2", "A", "current sensor: battery 2"),
    ("vol_acpwr", "V", "voltage sensor: aircraft power"),
    ("vol_outpwr", "V", "voltage sensor: system output power"),
    ("vol_bat_1", "V", "voltage sensor: battery 1"),
    ("vol_bat_2", "V", "voltage sensor: battery 2"),
    ("vol_res_p", "V", "voltage sensor: resolver board (+)"),
    ("vol_res_n", "V", "voltage sensor: resolver board (-)"),
    ("vol_back_p", "V", "voltage sensor: backplane (+)"),
    ("vol_back_n", "V", "voltage sensor: backplane (-)"),
    ("vol_gyro_1", "V", "voltage sensor: gyroscope 1"),
    ("vol_gyro_2", "V", "voltage sensor: gyroscope 2"),
    ("vol_acc_p", "V", "voltage sensor: INS accelerometers (+)"),
    ("vol_acc_n", "V", "voltage sensor: INS accelerometers (-)"),
    ("vol_block", "V", "voltage sensor: block"),
    ("vol_back", "V", "voltage sensor: backplane"),
    ("vol_srvo", "V", "voltage sensor: servos"),
    ("vol_cabt", "V", "voltage sensor: cabinet"),
    ("vol_fan", "V", "voltage sensor: cooling fan"),
]

for field, units, description in fields_data:
    SENSOR_DESCRIPTIONS.add_field(field, units, description)

# 添加传感器位置信息
sensor_positions_data = [
    ("Mag 1", "Tail stinger", -12.01, 0, 1.37, "Flt1002-1007"),
    ("Mag 2", "Front cabin, aft of cockpit", -0.60, -0.36, 0, "Flt1002-1007"),
    ("Mag 3", "Mid cabin, near INS", -1.28, -0.36, 0, "Flt1002-1007"),
    ("Mag 4", "Rear cabin, floor", -3.53, 0, 0, "Flt1002-1007"),
    ("Mag 5", "Rear cabin, ceiling", -3.79, 0, 1.20, "Flt1002-1007"),
    ("Flux A", "Mid cabin, near fuel tank", -3.27, -0.60, 0, "Flt1002-1007"),
    ("Flux B", "Tail, base of stinger", -8.92, 0, 0.96, "Flt1002-1007"),
    ("Flux C", "Rear cabin, port", -4.06, 0.42, 0, "Flt1002-1007"),
    ("Flux D", "Rear cabin, starboard", -4.06, -0.42, 0, "Flt1002-1007"),
    # Modified orientation for Flt1008 & Flt1009
    ("Mag 1", "tail stinger", -12.01, 0, 1.37, "Flt1008-1009"),
    ("Mag 2", "port diamond vertex, rear floor", -3.77, 0.60, 0.10, "Flt1008-1009"),
    ("Mag 3", "forward diamond vertex, rear floor", -3.17, 0, 0.11, "Flt1008-1009"),
    ("Mag 4", "starboard diamond vertex, rear floor", -3.77, -0.60, 0.12, "Flt1008"),
    ("Mag 4", "starboard diamond vertex, rear floor", -3.77, -0.35, 0.12, "Flt1009"),
    ("Mag 5", "aft diamond vertex, rear floor", -4.37, 0, 0.10, "Flt1008-1009"),
    ("Mag 6", "center of diamond, rear floor", -3.77, 0, 0.19, "Flt1008-1009"),
    ("Flux A", "front cabin, near battery", -0.73, -0.60, 0, "Flt1008-1009"),
    ("Flux B", "tail at base of stinger", -8.92, 0, 0.96, "Flt1008-1009"),
    ("Flux C", "mid cabin starboard side", -1.56, -0.55, 0, "Flt1008-1009"),
    ("Flux D", "mid-rear cabin, near fuel tank", -2.35, -0.55, 0, "Flt1008-1009"),
]

for sensor_data in sensor_positions_data:
    if len(sensor_data) == 5:  # 如果数据中只有5个值，则添加一个默认的 flight_number
        sensor_data = sensor_data + (" ",)
    sensor, description, x, y, z, flight_number = sensor_data
    SENSOR_DESCRIPTIONS.add_sensor_position(sensor, description, x, y, z, flight_number)


dataset_overview_note = Note(
    "Dataset Overview",
    [
        Note("HDF5 data fields for 2020 SGL flight data."),
        Note("Data sampled at 10Hz."),
        Note(
            "Includes magnetic sensors, inertial navigation system, avionics, and electrical currents and voltages."
        ),
        Note("(WGS-84) is altitude above WGS-84 ellipsoid."),
    ],
)

# 构建 Note 对象并添加到 data_description 实例
radar_note = Note(
    "radar",
    [
        Note(
            "unavailable at some times due to flown altitude exceeding instrument range"
        )
    ],
)

ins_pitch_roll_yaw_note = Note(
    "ins_pitch, ins_roll, ins_yaw",
    [
        Note(
            "convention: yaw clockwise from north, then pitch up, then roll to starboard"
        )
    ],
)

ogs_mag_alt_note = Note(
    "ogs_mag & ogs_alt",
    [
        Note("included for reference only"),
        Note("sampled from original Ontario Geological Survey flown in 2013"),
        Note(
            "unavailable at some times due to flown position not over original OGS survey"
        ),
    ],
)

ins_acc_xyz_note = Note(
    "ins_acc_x, ins_acc_y, ins_acc_z",
    [
        Note("AIRGrav (airborne gravimeter) system (http://www.sgl.com/Gravity.html)"),
        Note("3-axis gimballed INS, run in local-level, wander-angle mechanization"),
        Note("corrected for temperature, misalignment, and scale factor"),
        Note(
            "digitizers used for accelerometers are sigma-delta type and require low-pass filtering to increase resolution"
        ),
        Note("ins_acc_x: nominally horizontal, ccw from north by ins_wander [rad]"),
        Note("ins_acc_y: nominally horizontal, 90 [deg] ccw from ins_acc_x"),
        Note(
            "ins_acc_z: nominally vertical (up), includes gravity term (~navigation frame)"
        ),
    ],
)

ins_alt_note = Note(
    "ins_alt",
    [Note("combined GPS/inertial altitude computed in real time, contains lag error")],
)

lgtl_ltrl_nrml_acc_note = Note(
    "lgtl_acc, ltrl_acc, nrml_acc",
    [
        Note(
            "Garmin avionics module 010-0G600-00 (https://www.garmin.com/en-US/p/6427)"
        ),
        Note("nrml_acc does not include gravity term (body frame)"),
    ],
)

# 将 Note 对象添加到 data_description 实例
SENSOR_DESCRIPTIONS.add_field_notes("dataset_overview", dataset_overview_note)
SENSOR_DESCRIPTIONS.add_field_notes("radar", radar_note)
SENSOR_DESCRIPTIONS.add_field_notes(
    "ins_pitch, ins_roll, ins_yaw", ins_pitch_roll_yaw_note
)
SENSOR_DESCRIPTIONS.add_field_notes("ogs_mag & ogs_alt", ogs_mag_alt_note)
SENSOR_DESCRIPTIONS.add_field_notes("ins_acc_x, ins_acc_y, ins_acc_z", ins_acc_xyz_note)
SENSOR_DESCRIPTIONS.add_field_notes("ins_alt", ins_alt_note)
SENSOR_DESCRIPTIONS.add_field_notes(
    "lgtl_acc, ltrl_acc, nrml_acc", lgtl_ltrl_nrml_acc_note
)
